#include "mainwindow.h"
#include "dialog.h"
#include "vnc.h"
#include <stdlib.h>
#include <iostream>
#include <stdio.h>
#include <string.h>
#include <stdarg.h>
#include "utils.h"
#include <QTranslator>
#include <QPushButton>
#include <QComboBox>
#include <QDir>
#include "dlanguage.h"
#include "dclavier.h"
#include "dlock.h"
#include "dlock_setup.h"
#include <QtNetwork/QHostAddress>
#include <QtNetwork/QNetworkInterface>
#include <QtNetwork/QTcpSocket>
#include "dproxy.h"
#include "dsharewindows.h"
#include "dformatusb.h"
#include <QSettings>
#include "screenconfig.h"
#include "forceresolution.h"
#include "vnc.h"
#include "backup_raspi.h"
#include <QSpinBox>
#include <sys/statvfs.h>
#include <QByteArray>
#include <QStyleFactory>
#include <QLocale>


#include "QNetworkProxy"


MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),


    ui(new Ui::MainWindow)
{
    ///////////////////////////
    /// Theme application
    ///////////////////////////
    //QApplication::setDesktopSettingsAware(true);
    QApplication::setStyle("GTK+");
    /////////////////////////////////////////////


    ui->setupUi(this);
    config = new AppConfig();
    QIcon::setThemeName(Utils::getIconTheme());
    autostartChanged = false;
    languageChanged = false;
    needReboot = false;
    consoleChanged = false;
    tz_user_defined = false;
    kbd_user_defined = false;

    QProcess *process = new QProcess();
    process->start("pgrep sshd");
    process->waitForFinished();
    ui->chkB_ssh->setChecked(process->readAll() != "");

    string codeLocale = AppConfig::read_locale();
    char *nomLocale = get_locale_name(codeLocale.c_str());
    ui->pushB_language->setText(QString::fromUtf8(nomLocale));
    ui->pushB_Web->setText(tr("None"));

    ui->pushB_kbd->setText(QString::fromStdString(get_kbd_name()));

    AppConfig::get_autostart(ui->checkBox);
    get_autostart_info(ui->pushB_Web);

    watcher = new QFileSystemWatcher();
#ifndef __arm__
    watcher->addPath("/home/linutop/.config/appstarterrc");
    ui->grBox_SystemC->setTitle(tr("System configuration : PC"));
#else
    watcher->addPath("/home/pi/.config/appstarterrc");
    ui->grBox_SystemC->setTitle(tr("System configuration : Raspberry Pi"));
#endif

    connect(watcher,SIGNAL(fileChanged(QString)),this,SLOT(fileChanged(QString)));

    install = new LtopLogin();
    QString email,invoice;

    install->loadLogin(email,invoice);
    ui->lineEdit_email->setText(email);
    ui->lineEdit_invoice->setText(invoice);
    this->on_pushB_Login_clicked();


    ui->pushB_Format->setIcon(QIcon::fromTheme("edit-clear"));
    ui->pushB_System->setIcon(QIcon::fromTheme("edit-redo"));
    ui->pushB_USB->setIcon(QIcon::fromTheme("media-floppy"));
    ui->pushB_Config->setIcon(QIcon::fromTheme("network-wired"));
    ui->pushB_Help->setIcon(QIcon::fromTheme("gtk-help"));
    ui->buttonBox_CancelOk->button(QDialogButtonBox::Ok)->setIcon(QIcon::fromTheme("gtk-ok"));
    ui->buttonBox_CancelOk->button(QDialogButtonBox::Cancel)->setIcon(QIcon::fromTheme("gtk-cancel"));



    QFile file("/etc/timezone");
    if(file.open(QIODevice::ReadOnly | QIODevice::Text))
    {
        QTextStream line(&file);
        QString tmp = line.readAll();
        tmp.remove("\n");
        ui->pushB_time->setText(tmp);
    }

    if (get_wifi_iface().isEmpty())
    {
        ui->CHK_disable_wifi->setHidden(true);
        ui->CHK_disable_blue->setHidden(true);
    }


#ifdef __x86_64__
    QGroupBox *gb_security = new QGroupBox(tr("Choose in wich case give the root rights"),ui->tab_Security);

    rootOnly = new QRadioButton(tr("Root Only(Very secure)"));
    rootOrPiPasswd = new QRadioButton(tr("Root or ask pi password (Secure)"));
    rootOrPi = new QRadioButton(tr("Root or pi(Unsecure/default)"));
    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(rootOnly);
    vbox->addWidget(rootOrPiPasswd);
    vbox->addWidget(rootOrPi);
    vbox->addStretch(1);
    gb_security->setLayout(vbox);
    ui->verticalLayout_3->insertWidget(1,gb_security);

    rootOrPi->setChecked(true);
#endif

#ifdef __arm__
    //ui->pushB_Config->setDisabled(true);
    ui->pushB_Share->setHidden(true);
    //ui->pushB_Share->setDisabled(true);
    ui->chkBox_3->setHidden(true);
    ui->pushB_Force->setHidden(true);
   // ui->pushB_USB->setDisabled(true);
    ui->pushB_Format->setHidden(false);
    ui->chkB_no_hwclock->setHidden(true);
    ui->pushB_System->setHidden(true);

    ui->lbl_Infos->setHidden(true);

    QGroupBox *gb_security = new QGroupBox(tr("Choose in wich case give the root rights"),ui->tab_Security);

    rootOnly = new QRadioButton(tr("Root Only(Very secure)"));
    rootOrPiPasswd = new QRadioButton(tr("Root or ask pi password (Secure)"));
    rootOrPi = new QRadioButton(tr("Root or pi(Unsecure/default)"));
    QVBoxLayout *vbox = new QVBoxLayout;
    vbox->addWidget(rootOnly);
    vbox->addWidget(rootOrPiPasswd);
    vbox->addWidget(rootOrPi);
    vbox->addStretch(1);
    gb_security->setLayout(vbox);
    ui->verticalLayout_3->insertWidget(1,gb_security);

    rootOrPi->setChecked(true);
    blue_wifi_modified = false;
#else
    ui->pushB_extendMemory->setHidden(true);
    ui->F_disable->setHidden(true);
#endif

#ifdef DEMO
    this->setWindowTitle(this->windowTitle() + " DEMO");
    ui->checkBox->setDisabled(true);
    ui->pushB_lock->setDisabled(true);
#endif


    //diplay total disk memory
    char *total_ram;
    char buf[20];
    total_ram= readable_fs(get_mem_totale(),buf);
    ui->lbl_ram->setText(QString::fromUtf8(total_ram));

    //display @IP interface
    getaddressIPinterface();

    //display @IP
    getaddressIP();

    //get mac address
    getMacAdress();

    //get getaway address
    getGetawayAdress();

    //display  device
    ui->lbl_root->setText(get_root());

    root_device_info();

    getRebootSchedule();



    QSettings settings2(LOCKFILE,QSettings::IniFormat);
    if(settings2.value("lockEnabled",false).toBool())
    {
        ui->pushB_lock->setText(tr("Disable Linutop Lock"));
        ui->pushB_lock->setStyleSheet("color:rgb(255,0,0)");
        ui->pushB_lock->setIcon(QIcon::fromTheme("system-lock-screen"));
    }
    else
    {
        ui->pushB_lock->setText(tr("Enable Linutop Lock (reboot needed)"));
        ui->pushB_lock->setStyleSheet("color:rgb(0,125,0)");
        ui->pushB_lock->setIcon(QIcon::fromTheme("system-lock-screen"));
    }

    ///
    /// check if resolution was changed
    ///
    bool res_changed = false;
    QFile source("/home/pi/.config/autostart/lconfeditor.desktop");
    if(source.open(QFile::ReadOnly |
                  QFile::Text))
    {
        QTextStream in(&source);
        while (!in.atEnd())
       {
          QString line = in.readLine();
          QStringList list = line.split("=");
          if (list.at(0) == "chk_resolution" && list.at(1) == "true")
              res_changed = true;
       }
        source.close();
    }
    if (res_changed)
    {
        QString desktop;
        QFile source("/usr/share/applications/lconfeditor.desktop");
        if(source.open(QFile::ReadOnly |
                      QFile::Text))
        {
            QTextStream in2(&source);
            while (!in2.atEnd())
           {
              QString line = in2.readLine();
              QStringList list = line.split("=");
              if ((list.at(0) != "X-GNOME-Autostart-enabled") &&
                      (list.at(0) != "Hidden") &&
                  (list.at(0) != "chk_resolution"))
              {
                  desktop += line + "\n";
              }
           }

           source.close();

           QFile file2("/home/pi/.config/autostart/lconfeditor.desktop");
           desktop += "X-GNOME-Autostart-enabled=false\n";
           desktop += "Hidden=true\n";

           if(file2.open(QFile::WriteOnly | QIODevice::Truncate | QFile::Text))
           {
               QTextStream out2(&file2);
               out2 << desktop;
               file2.flush();
               file2.close();
           }
           ui->checkBox->setChecked(false);
        }
        sec = 15;
        QTimer *timer = new QTimer();
        connect(timer,SIGNAL(timeout()),this,SLOT(timer_timeout()));
        timer->start(1000);
        chk_res_message = new QMessageBox();
        chk_res_message->setWindowIcon(QIcon::fromTheme("video-display"));
        chk_res_message->setIcon(QMessageBox::Question);
        QPushButton *restoreButton = chk_res_message->addButton(tr("restore previous configuration"), QMessageBox::AcceptRole);
        chk_res_message->addButton(tr("Keep configuration"),QMessageBox::RejectRole);
        chk_res_message->setText(tr("<p><b>Would you like to keep this configuration ?</b></p>"
                          "<p>the previous configuration will be restored in ") + QString::number(sec) +
                          tr(" secondes if you do not answer to this question.<p>"));
        chk_res_message->exec();
        timer->stop();
        if (chk_res_message->clickedButton() == restoreButton) {
            QProcess *process = new QProcess();
            process->start("cp "SAVECONF " " PICONFFILE);
            process->waitForFinished();
            process->start("shutdown -r now");
            process->waitForFinished();
        }

    }
#ifdef __arm__
    QFile blacklist("/etc/modprobe.d/raspi-blacklist.conf");

    if(blacklist.open(QFile::ReadOnly |
                   QFile::Text))
    {
        bool parse = false;
        QTextStream in(&blacklist);
        while (!in.atEnd())
       {
          QString line = in.readLine();
          if (line == "#Linutop configuration")
              parse = true;
          QStringList list = line.split(" ");
          if (parse)
          {
              if (list.at(0) == "#wifi")
                  ui->CHK_disable_wifi->setChecked(true);
              if (list.at(0) == "#bluetooth")
                  ui->CHK_disable_blue->setChecked(true);
          }
          if(line == "#Linutop end")
              parse = true;
       }
        blacklist.close();
    }
#endif
}

void MainWindow::timer_timeout()
{
    sec -= 1;
    chk_res_message->setText(tr("<p><b>Would you like to keep this configuration ?</b></p>"
                     "<p>the previous configuration will be restored in ") + QString::number(sec) +
                     tr(" secondes if you do not answer to this question.<p>"));
    if(sec == 0)
    {
        chk_res_message->close();
        QProcess *process = new QProcess();
        process->start("cp "SAVECONF " " PICONFFILE);
        process->waitForFinished();
        process->start("shutdown -r now");
        process->waitForFinished();
    }
}


MainWindow::~MainWindow()
{
    delete ui;
}


//free Disk space


//RAM

size_t MainWindow::get_mem_totale()
{
    FILE *f;
    char line[256];
    size_t memtotal = 0;

    if ((f = fopen ("/proc/meminfo", "r")) == NULL)
       return 0;

    while (fgets (line, 255, f))
    {
        char name[64];
        size_t l;
        int ret;
        ret = sscanf (line, "%s %zu", name, &l);
        if ((ret == 2) && (strcmp (name, "MemTotal:") == 0))
        {
            memtotal = l;
            break;
        }
    }
    fclose (f);

    return memtotal * 1024 ;
}



 //human readable file size
char*  MainWindow::readable_fs(double size/*in bytes*/, char *buf) {
   int i = 0;
    const char* units[] = {"B", "KB", "MB", "GB", "TB", "PB", "EB", "ZB", "YB"};
    while (size >= 1024) {
        size /= 1024;
        i++;
    }

    sprintf(buf, "%.2f %s", size, units[i]);
    return buf;
}






// open help documentation
void MainWindow::on_pushB_Help_clicked()
{
    QLocale loc = QLocale();
    if (loc.language() == QLocale::French)
        Utils::runCmd("yelp /usr/share/gnome/help/linutopos/fr/linutopos.xml");
    else
        Utils::runCmd("yelp /usr/share/gnome/help/linutopos/C/linutopos.xml");
}





void MainWindow::on_buttonBox_CancelOk_accepted()
{
    install->saveLogin(ui->lineEdit_email->text(),ui->lineEdit_invoice->text());


    const char *temp = ui->pushB_language->text().toStdString().c_str();
    config->locale_value = QString::fromUtf8(get_code_name(temp));
    config->tz = ui->pushB_time->text();
    config->writeConfig(timeZoneChanged);
    config->set_autostart(ui->checkBox,autostartChanged);
#ifdef __arm__
    if (blue_wifi_modified)
    {
        config->set_wifi_blue(ui->CHK_disable_wifi->isChecked(), ui->CHK_disable_blue->isChecked());
        needReboot = true;
    }
#endif


    //scheduler reboot
    setRebootSchedule();


#ifndef __arm__
    if(consoleChanged)
    {
        if(ui->chkBox_2->isChecked())
        {
            QProcess::startDetached("sh" LIBEXECDIR "/arm_console_mode true");
            needReboot =true;
        }
        else
        {
            QProcess::startDetached("sh" LIBEXECDIR "/arm_console_mode false");
        }
    }
#else

#endif

    if(ui->chkB_ssh->isChecked())
#ifdef __arm__
        QProcess::startDetached("/etc/init.d/ssh restart");
    else
        QProcess::startDetached("/etc/init.d/ssh stop");
#else
        QProcess::startDetached("sudo start ssh");
    else
        QProcess::startDetached("sudo stop ssh");
#endif

    //ask for reboot if needed
    if(needReboot)
        rebootNeeded();
}



void MainWindow::on_checkBox_stateChanged(int)
{
    autostartChanged = true;
}

void MainWindow::fileChanged(QString)
{
    get_autostart_info(ui->pushB_Web);
    //watcher->addPath(path);
}
/////////////////////////////////////////////////////////////
/// General onglet
/////////////////////////////////////////////////////////////
void MainWindow::on_pushB_language_clicked()
{
    DLanguage *l = new DLanguage();
    connect(l, SIGNAL(signalUpdateBtn(QString,bool)), this, SLOT(updatePushB_language(QString,bool)));
    l->setWindowIcon(QIcon::fromTheme("preferences-desktop-locale"));
    l->populate_locale_tree(ui->pushB_language);
    l->exec();
}

void MainWindow::updatePushB_language(QString language, bool cancel)
{
    if(cancel)
    {
        string codeLocale = AppConfig::read_locale();
        char *nomLocale = get_locale_name(codeLocale.c_str());
        ui->pushB_language->setText(QString::fromUtf8(nomLocale));
    }
    else
    {
        ui->pushB_language->setText(language);
        needReboot = true;
        languageChanged = true;
        update_locale_settings();
    }
}

void MainWindow::on_pushB_kbd_clicked()
{
    DClavier *d = new DClavier();
    connect(d, SIGNAL(signalUpdateBtn(QString)), this, SLOT(updatePushB_Kbd(QString)));
    d->setWindowIcon(QIcon::fromTheme("preferences-desktop-keyboard"));
    d->populate_kbd_tree(ui->pushB_kbd);
    d->exec();
}

void MainWindow::updatePushB_Kbd(QString kbd)
{
    ui->pushB_kbd->setText(kbd);
    kbd_user_defined = true;
}
/* if the tz and kbd aren't defined by the user, update them for the selected language */
void MainWindow::update_locale_settings()
{
    if (kbd_user_defined && tz_user_defined)
        return;

    QString locale = ui->pushB_language->text().toStdString().c_str();

    string tz,l,kbd;
    get_from_language(locale.toStdString().c_str(), l, kbd, tz);
    if (!kbd_user_defined)
    {
        //get this part: en_>>GB<<.UTF8 of the UTF8 code, lower  it and parse KBDFILE to get the name of the keyboard associated
        QString code = QString::fromStdString(get_code_name(locale.toStdString().c_str())).split("_")[1].split(".")[0].toLower();
        if(kbd != "")
            code = QString::fromStdString(kbd);

        QString name = get_kbd_name_from_code(code);
        if(name != "")
            ui->pushB_kbd->setText(name);

        QString cmd = "sh /usr/lib/ltools/setkbd.sh " + code;
        Utils::runCmd(cmd);

        /* TODO: put in script setkbd.sh - re-use in dclavier.cpp...*/
    #ifdef __arm__
        QFile file("/home/pi/.config/autostart/setkbd.desktop");
    #else
        QFile file("/home/linutop/.config/autostart/setkbd.desktop");
    #endif
        if(file.open(QIODevice::ReadWrite))
        {
            file.resize(0);
            QByteArray content;
            content.append("[Desktop Entry]\n");
            content.append("Type=Application\n");
            content.append("Exec=setxkbmap " + code +"\n");
            content.append("Name=Keyboard_setting");
            file.write(content);
        }
        file.close();
    }

    if (!tz_user_defined)
    {
        ui->pushB_time->setText(QString::fromStdString(tz));
    }

}

void MainWindow::on_pushB_time_clicked()
{
    Dialog *time = new Dialog();
    connect(time,SIGNAL(signalUpdateBtn(QString)),this,SLOT(updatePushB_tz(QString)));
    time->populate_tz_tree(ui->pushB_time);
    time->exec();
}

void MainWindow::on_pushB_Screen_clicked()
{
    #if __arm__
        ScreenConfig *sc = new ScreenConfig();
        sc->setWindowIcon(QIcon::fromTheme("video-display"));
        sc->exec();
    #else
        Utils::runCmd("xfce4-display-settings");
    #endif
}



void MainWindow::updatePushB_tz(QString tz){

    ui->pushB_time->setText(tz);
    needReboot = true;
    timeZoneChanged = true;
    tz_user_defined = true;
}

void MainWindow::on_pushB_Force_clicked()
{
    ForceResolution *f = new ForceResolution();
    f->setWindowIcon(QIcon::fromTheme("video-display"));
    f->exec();
}
void MainWindow::on_pushB_Web_clicked()
{
    Utils::runCmd("appstarter");
}







//////////////////////////////////////////////
/// Install onglet
//////////////////////////////////////////////

void MainWindow::on_pushB_Login_clicked()
{
    QFile filemode("/tmp/choixmodeproxy");
    if(filemode.open(QIODevice::ReadOnly| QIODevice::Text)){
        QTextStream choix(&filemode);
        int tmp = choix.readAll().toInt();
        Dproxy::recup_mode=tmp;
        
        if(Dproxy::recup_mode==2){
            QString tmp = Dproxy::recup_host;
            const char* url= tmp.toUtf8().data();
            if(Dproxy::recup_type=="http"){
                setenv("http_proxy", url, 1);
                //qDebug()<<"Environment var http_proxy:" << getenv("http_proxy");
            }else if(Dproxy::recup_type == "ftp"){
                setenv("ftp_proxy", url, 1);
                //        qDebug()<<"Environment var ftp_proxy" << getenv("ftp_proxy");
            }else if(Dproxy::recup_type == "https"){
                setenv("https_proxy", url, 1);
                //        qDebug()<<"Environment var https_proxy" << getenv("https_proxy");
            }
        }else{
            setenv("http_proxy", "", 1);
            setenv("ftp_proxy", "", 1);
            setenv("https_proxy", "", 1);
        }
    }
   install->clearData();
   connect(install,SIGNAL(populateInstallTree(QVector<QJsonObject>)),this,SLOT(populateInstallTree(QVector<QJsonObject>)));
   install->_get_json(ui->lineEdit_email->text(),ui->lineEdit_invoice->text());

}

void MainWindow::on_lineEdit_email_returnPressed()
{
    on_pushB_Login_clicked();
}

void MainWindow::on_lineEdit_invoice_returnPressed()
{
    on_pushB_Login_clicked();
}

void MainWindow::on_pushB_Install_clicked()
{
    if(ui->list_Install->currentIndex().isValid())
    {
        int index = ui->list_Install->currentIndex().row();
        install->applyConfig(ui->lineEdit_email->text(),ui->lineEdit_invoice->text(),index);
    }
}

void MainWindow::on_list_Install_doubleClicked(const QModelIndex)
{
    on_pushB_Install_clicked();
}

void MainWindow::on_list_Install_currentRowChanged(int currentRow)
{
    QString desc;
    ui->pushB_Install->setEnabled(1);
    if(install->services[currentRow]["type"].toInt() == 0)
    {
        ui->pushB_Install->setText(tr("Install"));
        ui->lbl_Install_Type->setText(tr("<b>Linutop OS download:</b>"));
        desc = install->services[currentRow]["desc"].toString();
    }
    else
    {
        ui->pushB_Install->setText(tr("Apply"));
        ui->lbl_Install_Type->setText(tr("<b>System configuration:</b>"));
        if(install->services[currentRow]["longdesc"].isUndefined())
            desc = install->services[currentRow]["desc"].toString();
        else
            desc = install->services[currentRow]["longdesc"].toString();
    }

    ui->lbl_Install->setText(desc);
}


void MainWindow::populateInstallTree(QVector<QJsonObject> services)
{
    ui->list_Install->selectionModel()->reset();
    ui->list_Install->clear();
    for(int i=0; i<services.size(); i++)
    {
        QListWidgetItem *item = new QListWidgetItem();
        item->setText(services[i]["desc"].toString());
        if(services[i]["type"].toInt() == 0)
            item->setIcon(QIcon::fromTheme("system-software-install"));
        else
            item->setIcon(QIcon::fromTheme("gnome-control-center"));
        ui->list_Install->addItem(item);
    }
}



/////////////////////////////////////////////////////////////
/// Network onglet
/////////////////////////////////////////////////////////////
///
void MainWindow::on_pushB_Config_clicked()
{

    Utils::runCmd("nm-connection-editor");
}
void MainWindow::on_pushB_Share_clicked()
{
    Utils::runCmd("system-config-samba");
}

void MainWindow::on_pushB_Windows_clicked()
{
  DshareWindows *d = new DshareWindows();
  d->setWindowIcon(QIcon::fromTheme("preferences-system-network"));
  d->exec();

}


void MainWindow::on_pushB_config_clicked()
{
#ifdef __arm__
    vnc *win=new vnc();
    win->exec();
#else
   // Utils::runCmd("gx11vnc");
    vnc *win=new vnc();
    win->exec();
#endif


}

//Proxy settings window
void MainWindow::on_pushB_proxy_clicked()
{
   Dproxy *d = new Dproxy();
   d->setWindowIcon(QIcon::fromTheme("preferences-system-network"));
   d->exec();
}









/////////////////////////////////////////////////
/// Advanced onglet
/////////////////////////////////////////////////

void MainWindow::on_pushB_USB_clicked()
{

    Backup_raspi *d = new Backup_raspi();
    d->exec();

}

void MainWindow::setRebootSchedule()
{
    QString newFile;
    QFile file("/etc/crontab");
    if(file.open(QFile::ReadWrite |
                  QFile::Text))
    {
        bool keep = true;
        QTextStream in(&file);
        while (!in.atEnd())
        {
          QString line = in.readLine();
          if(line == "#Linutop schedule configuration")
              keep = false;
          if(keep)
              newFile += line + "\n";
          if(line == "#Linutop end")
              keep = true;
        }
        file.close();

        newFile += "#Linutop schedule configuration\n";

        if(!ui->chkB_sched_reboot->isChecked())
            newFile += "#";
        else
            QProcess::startDetached("service cron restart");

        newFile += ui->spinBox_minute->text() + " " + ui->spinBox_hour->text() + "  * * * ";
        newFile += "root reboot\n";
        newFile += "#Linutop end\n";

        if(file.open(QFile::WriteOnly | QIODevice::Truncate |
                      QFile::Text))
        {
            QTextStream out(&file);
            out << newFile;
            file.flush();
            file.close();
        }
    }

    QString newFile2;
    QFile file2("/etc/rc.local");
    if(file2.open(QFile::ReadWrite |
                  QFile::Text))
    {
        newFile2 += "#Linutop schedule configuration\n";
        if(ui->chkB_sched_reboot->isChecked())
            newFile2 += "start cron\n";
        newFile2 += "#Linutop end\n";

        bool keep2 = true;
        QTextStream in2(&file2);
        while (!in2.atEnd())
        {
          QString line2 = in2.readLine();
          if(line2 == "#Linutop schedule configuration")
              keep2 = false;
          if(keep2)
              newFile2 += line2 + "\n";
          if(line2 == "#Linutop end")
              keep2 = true;
        }
        file2.close();

        if(file2.open(QFile::WriteOnly | QIODevice::Truncate |
                      QFile::Text))
        {
            QTextStream out2(&file2);
            out2 << newFile2;
            file2.flush();
            file2.close();
        }
    }
}



void MainWindow::getRebootSchedule()
{
    QFile file("/etc/crontab");
    if(file.open(QFile::ReadOnly |
                  QFile::Text))
    {
        bool parse = false;
        QTextStream in(&file);
        while (!in.atEnd())
        {
            QString line = in.readLine();
            if(line == "#Linutop end")
                parse = false;
            if(line == "#Linutop schedule configuration")
                parse = true;

            QStringList list = line.split(" ");
            if(list.size() > 0 && parse)
            {
                QString minute = list.at(0);

                if(minute.at(0) == '#')
                    minute.remove("#");
                else
                    ui->chkB_sched_reboot->setChecked(true);

                ui->spinBox_minute->setValue(minute.toInt());
                ui->spinBox_hour->setValue(list.at(1).toInt());
            }
        }
        file.close();
    }
}


void MainWindow::on_pushB_System_clicked()
{

    QMessageBox::StandardButton restore_dialog;
    restore_dialog = QMessageBox::information(this, " ","Would you like to restore your system now?\nThe system will reboot if you continue.\n\nA menu will offer you to select the restoration mode.", QMessageBox::Yes|QMessageBox::No);


    if (restore_dialog == QMessageBox::Yes) {


        QString cmd ="sh " LIBEXECDIR "/restore.sh restore";
        Utils::runCmd(cmd);

        Utils::runCmd("shutdown -r now");
      // restore_dialog.destroy(true);
    }

}

void MainWindow::on_pushB_Format_clicked()
{
    DformatUSB *d = new DformatUSB();
    d->exec();
   // Utils::runCmd("lusbclean");
}

void MainWindow::set_reboot_sched(bool sched, int hour,  int minute){

    sched=ui->chkB_sched_reboot->isChecked();
    hour=ui->spinBox_hour->value();
    minute=ui->spinBox_minute->value();
}




void MainWindow::on_chkB_no_hwclock_clicked()
{
    if(ui->chkB_no_hwclock->isChecked())
    { Utils::runCmd("nohwclock");}
}



void MainWindow::on_chkBox_2_clicked()
{
    consoleChanged = true;
}
#include <unistd.h>
void MainWindow::on_pushB_extendMemory_clicked(){
    QMessageBox::StandardButton extend_dialog;
    extend_dialog = QMessageBox::information(this, " ", "Would you like to extend the memory to its full capacity?", QMessageBox::Yes|QMessageBox::No);
    if(extend_dialog == QMessageBox::Yes) {
        QString cmd ="sh " LIBEXECDIR "/expandmemory";

        progress_dialog = new QProgressDialog(this);
        QProcess *proc = new QProcess();
        progress_dialog->setWindowTitle("Expanding");
        progress_dialog->setLabelText("Expanding memory, please wait...");
        progress_dialog->setRange(0,0);
        progress_dialog->setCancelButtonText(QString());
        progress_dialog->setWindowModality(Qt::WindowModal);
        connect(proc, SIGNAL(finished(int, QProcess::ExitStatus)), this, SLOT(progress_timeout(int,QProcess::ExitStatus)));

        proc->start(cmd);
        progress_dialog->exec();
        proc->waitForFinished();
        rebootNeeded();
    }
}

void MainWindow::progress_timeout(int nb, QProcess::ExitStatus st)
{
    progress_dialog->cancel();
}


////////////////////////////////////////////////////
/// Security onglet
////////////////////////////////////////////////////

void MainWindow::on_pushB_Setup_clicked()
{
    DLock_setup *d = new DLock_setup();
    d->setWindowIcon(QIcon::fromTheme("system-lock-screen"));
    d->exec();
}


// Apply Lock system
void MainWindow::on_pushB_lock_clicked()
{
    QSettings settings(LOCKFILE,QSettings::IniFormat);
    if(!settings.value("lockEnabled",false).toBool())
    {
        DLock *d = new DLock();
        d->setWindowIcon(QIcon::fromTheme("system-lock-screen"));
        if(d->exec() == QDialog::Accepted){
#ifdef __arm__
            if(rootOnly->isChecked())
                config->lockMode = "-r " + d->password;
            else if(rootOrPiPasswd->isChecked())
                config->lockMode = "-s";
            else if(rootOrPi->isChecked())
                config->lockMode = "-u";
#endif

#ifdef __x_86_64__
            if(rootOnly->isChecked())
                config->lockMode = "-r " + d->password;
            else if(rootOrPiPasswd->isChecked())
                config->lockMode = "-s";
            else if(rootOrPi->isChecked())
                config->lockMode = "-u";
#endif

            QSettings settings(LOCKFILE,QSettings::IniFormat);
            settings.setValue("lockEnabled",true);
            ui->pushB_lock->setText(tr("Disable Linutop Lock"));
            ui->pushB_lock->setStyleSheet("color:rgb(255,0,0)");
            ui->pushB_lock->setIcon(QIcon::fromTheme("system-lock-screen"));

            QMessageBox::StandardButton reboot_dialog;
            reboot_dialog = QMessageBox::warning(this, tr("Reboot needed"), tr("Your system needs to be rebooted in order to apply the changes\n\nWould you like to reboot now ?"), QMessageBox::Yes|QMessageBox::No);

            if (reboot_dialog == QMessageBox::Yes) {
                config->writeConfig(timeZoneChanged);
                Utils::runCmd("shutdown -r now");
            }
        }
        else
        {
            needReboot = true;
        }
    }
    else
    {
        ui->pushB_lock->setText(tr("Enable Linutop Lock (reboot needed)"));
        ui->pushB_lock->setStyleSheet("color:rgb(0,125,0)");
        ui->pushB_lock->setIcon(QIcon::fromTheme("system-lock-screen"));

        QSettings settings(LOCKFILE,QSettings::IniFormat);
        settings.setValue("lockEnabled",false);
    }
}



void MainWindow::rebootNeeded()
{
    QMessageBox::StandardButton ask_shutdown;
        ask_shutdown = QMessageBox::warning(this, NULL, "Your system needs to reboot in order to apply the changes.<br/>Would you like to reboot now ?",
                                      QMessageBox::Yes|QMessageBox::No);
        if (ask_shutdown == QMessageBox::Yes) {
            Utils::runCmd("sudo shutdown -r 0");
        }
}

///////////////
/// About onglet
///////////////


//root device infos

void MainWindow::root_device_info(){

    QProcess process;
    process.start("sh", QStringList() << "-c" << "df -h | grep ^/dev");
    process.waitForFinished();
    QString ret = process.readAll();

    ret.remove("\n");

    QStringList lstRet = ret.split(" ",QString::SkipEmptyParts);

    ui->lbl_root_info->setText("(" + lstRet.at(3) + " " + "/" + " " + lstRet.at(1) + ")");

}



//get IP address of the system
void MainWindow::getaddressIP(){

    QStringList addresses;
      foreach(QHostAddress address, QNetworkInterface::allAddresses())
      {
        // Filtre les adresses localhost ...
        if(address != QHostAddress::LocalHostIPv6
           && address != QHostAddress::LocalHost
           // ... APIPA ...
           && !address.isInSubnet(QHostAddress::parseSubnet("169.254.0.0/16"))
           // ...  Local IPv6 link
           && !address.isInSubnet(QHostAddress::parseSubnet("FE80::/64")))
        {
          addresses << address.toString();
        }
      }


      foreach(QString address, addresses)
      {
          ui->lbl_ip->setText(address);
      }

}

QString MainWindow::get_root()
{
    QString rootD;
    #ifdef __arm__
        QFile file(ROOTFILE);

        if(file.open(QIODevice::ReadOnly|QIODevice::Text))
        {
            QTextStream flux(&file);
            QString line;
            line=flux.readAll();
            QStringList list = line.split(" ");
            foreach (QString tmp, list) {
                if(tmp.split("=").at(0) =="root")
                    rootD=tmp.split("=").at(1);
           }
        }
        return rootD;
    #else
        QProcess process;
        process.start("sh", QStringList() << "-c" << "df -h | grep /$");
        process.waitForFinished();
        QString ret = process.readAll();
        ret.remove("\n");
        QStringList lstRet = ret.split(" ",QString::SkipEmptyParts);
        rootD=lstRet.at(0);
        return rootD;
    #endif
}



//get MAC address of the system
void MainWindow::getMacAdress()
{
    QProcess process;
    process.start("sh", QStringList() << "-c" << "ifconfig | grep HWaddr");
    process.waitForFinished();
    QString ret = process.readAll();
    ret.remove("\n");
    if(ret.split("HWaddr ").size() > 1)
        ui->lbl_macAddress->setText(ret.split("HWaddr ").at(1));
    else
        ui->lbl_macAddress->setText("none");
}

//get getaway address of the system
void MainWindow::getGetawayAdress()
{
    QProcess process;
    process.start("sh", QStringList() << "-c" << "ip route | grep default");
    process.waitForFinished();
    QString ret = process.readAll();
    ret.remove("\n");
    QString str;
    if(ret.split(" ").size() > 2)
        str = ret.split(" ").at(2);
    else
        str = "none";
    ui->lbl_getaway->setText(str);
}


//get IP adress
void MainWindow::getaddressIPinterface(){

    QProcess process;
    process.start("sh", QStringList() << "-c"<<"ifconfig ");
    process.waitForFinished();
    QString ret = process.readAll();


    ret.remove("\n");
    if(ret.isEmpty())
        ret = "none";
    ui->lbl_network->setText(ret.split(" ").at(0)+" => ");

}

void MainWindow::on_CHK_disable_blue_clicked()
{
    blue_wifi_modified = true;
}

void MainWindow::on_CHK_disable_wifi_clicked()
{
    blue_wifi_modified = true;
}

QByteArray MainWindow::get_wifi_iface()
{
    QProcess ifconfig_proc;
    ifconfig_proc.start("bash", QStringList() << "-c" << "ifconfig | grep wlan | cut -d ' ' -f 1");
    ifconfig_proc.waitForFinished();
    return (ifconfig_proc.readAll().simplified());
}


