#include "screenconfig.h"
#include <QFile>
#include <QTextStream>
#include <QMessageBox>
#include <QProcess>
#include <QPushButton>
#include <QDebug>

#include <dadv_overscan.h>

ScreenConfig::ScreenConfig(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ScreenConfig)
{
    ui->setupUi(this);
    setWindowTitle(tr("Display configuration"));
    ui->lblTitreIcon->setPixmap(QIcon::fromTheme("video-display").pixmap(50,50));
    readConfig();

    if (ui->CBRes->currentIndex() != 0)
        choosed_res = ui->CBRes->currentText().split(" ").at(0);
    else
        choosed_res = "0";
}

ScreenConfig::~ScreenConfig()
{
    delete ui;
}

void ScreenConfig::on_buttonBox_accepted()
{
    QString newFile;

    if (QFile::exists(SAVECONF))
    {
        QFile::remove(SAVECONF);
    }
    QFile::copy(PICONFFILE, SAVECONF);

    QFile file(PICONFFILE);
    if(file.open(QFile::ReadWrite |
                  QFile::Text))
    {
        bool keep = true;
        QTextStream in(&file);
        while (!in.atEnd())
        {
          QString line = in.readLine();
          if(line == "#Linutop Display configuration")
              keep = false;
          if(keep)
              newFile += line + "\n";
          if(line == "#Linutop end")
              keep = true;
        }
        file.close();

        newFile += "#Linutop Display configuration\n";
        newFile += "sdtv_mode=" + QString::number(ui->CBMode->currentIndex()) + "\n";
        newFile += "sdtv_aspect=" + QString::number(ui->CBAspect->currentIndex() +1) + "\n";
        newFile += "display_rotate=";
        switch( ui->CBTransfo->currentIndex() )
        {
        case 0:
            newFile += "0";
            break ;
        case 1 :
            newFile += "1";
            break ;
        case 2 :
            newFile += "2";
            break ;
        case 3:
            newFile += "3";
            break ;
        case 4:
            newFile += "0x10000";
            break ;
        case 5:
            newFile += "0x20000";
            break ;
        }
        newFile += "\n";

            if(ui->CBRes->currentIndex() != 0)
            {
                newFile += "hdmi_group=2\n";
                newFile += "hdmi_mode=";
                switch( ui->CBRes->currentIndex() )
                {
                case 1 :
                    newFile += "73";
                    break ;
                case 2 :
                    newFile += "82";
                    break ;
                case 3:
                    newFile += "69";
                    break ;
                case 4:
                    newFile += "58";
                    break ;
                case 5:
                    newFile += "83";
                    break ;
                case 6:
                    newFile += "51";
                    break ;
                case 7:
                    newFile += "47";
                    break ;
                case 8:
                    newFile += "42";
                    break ;
                case 9:
                    newFile += "81";
                    break ;
                case 10:
                    newFile += "32";
                    break ;
                case 11:
                    newFile += "16";
                    break ;
                case 12:
                    newFile += "9";
                    break ;
                case 13:
                    newFile += "4";
                    break ;
                case 14:
                    newFile += "87\n";
                    newFile += "hdmi_ignore_edid=0xa5000080\n";
                    newFile += "hdmi_cvt 3840 2160 15\n";
                    break ;
                }
                newFile += "\n";
            }
        if(ui->CBmem->currentIndex() != 0)
        {
            QString mem = ui->CBmem->currentText();

            if(mem.toInt() > 192)
                newFile += "gpu_mem_256=192\n";
            else
                newFile += "gpu_mem_256=" + mem + "\n";

            if(mem.toInt() > 448)
                newFile += "gpu_mem_512=448\n";
            else
                newFile += "gpu_mem_512=" + mem + "\n";

            if(mem.toInt() > 944)
                newFile += "gpu_mem_1024=944\n";
            else
                newFile += "gpu_mem_1024=" + mem + "\n";

            newFile += "gpu_mem=" + mem + "\n";
        }
        if(ui->CBDepth->currentIndex() != 0)
        {
            switch( ui->CBDepth->currentIndex() )
            {
            case 1 :
                newFile += "framebuffer_depth=24\n";
                break ;
            case 2 :
                newFile += "framebuffer_depth=32\n";
                newFile += "framebuffer_ignore_alpha=1\n";
                break ;
            default:
                break;
            }
        }
        if(ui->chkSound->isChecked())
        {
            newFile += "hdmi_drive=2\n";
        }

        newFile += "#Linutop end\n";

        if(file.open(QFile::WriteOnly | QIODevice::Truncate |
                      QFile::Text))
        {
            QTextStream out(&file);
            out << newFile;
            file.flush();
            file.close();
        }

        ///////////////////////////
        /// configuration .desktop
        ///////////////////////////

        QString desktop;
        QFile source("/usr/share/applications/lconfeditor.desktop");
        if(source.open(QFile::ReadOnly |
                      QFile::Text))
        {
            QTextStream in2(&source);
            while (!in2.atEnd())
           {
              QString line = in2.readLine();
              QStringList list = line.split("=");
              if ((list.at(0) != "X-GNOME-Autostart-enabled") &&
                  (list.at(0) != "Hidden"))
              {
                  desktop += line + "\n";
              }
           }

           source.close();

           QFile file2("/home/pi/.config/autostart/lconfeditor.desktop");
           desktop += "X-GNOME-Autostart-enabled=true\n";
           desktop += "Hidden=false\n";
           desktop += "chk_resolution=true\n";

           if(file2.open(QFile::WriteOnly | QIODevice::Truncate | QFile::Text))
           {
               QTextStream out2(&file2);
               out2 << desktop;
               file2.flush();
               file2.close();
           }
        }

    }
    QMessageBox::StandardButton ask_shutdown;
        ask_shutdown = QMessageBox::warning(this, NULL, "Previous configuration have been saved into /boot/config.save.<br/>You will be asked to confirm your resolution at startup.<br/><br/>Your system needs to reboot in order to aply the changes.<br/>Would you like to reboot now ?",
                                      QMessageBox::Yes|QMessageBox::No);
        if (ask_shutdown == QMessageBox::Yes) {
            QProcess::startDetached("sudo shutdown -r 0");
        } else {

        }

    close();
}

void ScreenConfig::readConfig()
{
    QFile file(PICONFFILE);
    if(file.open(QFile::ReadOnly |
                  QFile::Text))
    {
        bool custom_res = false;
        QTextStream in(&file);
        while (!in.atEnd())
        {
            QString line = in.readLine();

            QStringList list = line.split("=");
            if((list.at(0) == "sdtv_mode"))
            {
                ui->CBMode->setCurrentIndex(list.at(1).toInt());
            }
            if((list.at(0) == "sdtv_aspect"))
            {
                ui->CBAspect->setCurrentIndex(list.at(1).toInt() -1);
            }
            if((list.at(0) == "display_rotate"))
            {
                switch(list.at(1).toInt())
                {
                case 0:
                    ui->CBTransfo->setCurrentIndex(0);
                    break ;
                case 1 :
                    ui->CBTransfo->setCurrentIndex(1);
                    break ;
                case 2 :
                    ui->CBTransfo->setCurrentIndex(2);
                    break ;
                case 3:
                    ui->CBTransfo->setCurrentIndex(3);
                    break ;
                default:
                    if(list.at(1) == "0x10000")
                        ui->CBTransfo->setCurrentIndex(4);
                    if(list.at(1) == "0x20000")
                        ui->CBTransfo->setCurrentIndex(5);
                    break;
                }
            }
            if((list.at(0) == "hdmi_mode"))
            {
                switch(list.at(1).toInt())
                {
                case 0 :
                    break ;
                case 73 :
                    ui->CBRes->setCurrentIndex(1);
                    break ;
                case 82 :
                    ui->CBRes->setCurrentIndex(2);
                    break ;
                case 69:
                    ui->CBRes->setCurrentIndex(3);
                    break ;
                case 58:
                    ui->CBRes->setCurrentIndex(4);
                    break ;
                case 83:
                    ui->CBRes->setCurrentIndex(5);
                    break ;
                case 51:
                    ui->CBRes->setCurrentIndex(6);
                    break ;
                case 47:
                    ui->CBRes->setCurrentIndex(7);
                    break ;
                case 42:
                    ui->CBRes->setCurrentIndex(8);
                    break ;
                case 81:
                    ui->CBRes->setCurrentIndex(9);
                    break ;
                case 32:
                    ui->CBRes->setCurrentIndex(10);
                    break ;
                case 16:
                    ui->CBRes->setCurrentIndex(11);
                    break ;
                case 9:
                    ui->CBRes->setCurrentIndex(12);
                    break ;
                case 4:
                    ui->CBRes->setCurrentIndex(13);
                    break ;
                default:
                    break;
                }
            }
            if (line == "hdmi_cvt 3840 2160 15")
                ui->CBRes->setCurrentIndex(14);
            if((list.at(0) == "framebuffer_depth"))
            {
                switch(list.at(1).toInt())
                {
                case 24 :
                    ui->CBDepth->setCurrentIndex(1);
                    break ;
                case 32 :
                    ui->CBDepth->setCurrentIndex(2);
                    break ;
                default:
                    break;
                }
            }
            if((list.at(0) == "gpu_mem_1024"))
            {
                for(int i=1; i<ui->CBmem->count(); i++)
                {
                    if(ui->CBmem->itemText(i) == list.at(1))
                        ui->CBmem->setCurrentIndex(i);
                }
            }
            if((list.at(0) == "hdmi_drive") && (list.at(1) == "2"))
            {
                ui->chkSound->setChecked(true);
            }
        }
        file.close();
    }
}

void ScreenConfig::on_buttonBox_rejected()
{
    close();
}

void ScreenConfig::on_PB_adv_overscan_clicked()
{
    dadv_overscan *sc = new dadv_overscan(this);
    sc->setWindowIcon(QIcon::fromTheme("video-display"));
    sc->exec();
    if (sc->custom_res)
        ui->CBRes->setCurrentIndex(0);
}

void ScreenConfig::on_CBRes_currentIndexChanged(int index)
{
    if (index != 0)
        choosed_res = ui->CBRes->currentText().split(" ").at(0);
    else
        choosed_res = "0";
}
