#include "utils.h"
#include "includes/locales.cpp"
#include <iostream>
#include "dlock_setup.h"
#include <QMessageBox>

#define CURRENTKBDFILE "/etc/default/keyboard"
#define KBDFILE "/usr/share/X11/xkb/rules/base.lst"

Utils::Utils()
{
}


/* run an asynchrone command */
void Utils::runCmd(QString command, int timems)
{
    QProcess *proc = new QProcess();
    proc->start(command);
    proc->waitForFinished(timems);
}


/* get locale data from its name */
void
get_from_language (const char *locale, string &pkgs, string &kdb, string &tz)
{
    int i;
    bool found = false;
    for (i=0; locale_data[i].name != NULL; i++)
    {
        if (strncmp (locale, locale_data[i].name, strlen (locale)) == 0)
        {
            kdb = strdup(locale_data[i].kbd);
            pkgs = strdup(locale_data[i].pkgs);
            tz = strdup(locale_data[i].tz);
            found = true;
            break;
        }
    }
    /* we shouldn't get here */
    if (!found)
    {
        kdb = "";
        pkgs = "";
        tz = "";
    }
}


/* get locale code from its code */
char *
get_locale_name (const char *code)
{
    char *name = NULL;
    int count = 0;
    while (locale_data[count].code != '\0') {
        if (!strncmp (locale_data[count].code, code, strlen (code))) {
            name = strdup (locale_data[count].name);
            return name;
        }
        count++;
    }
    return name;
}


/* get locale name from its name */
char *
get_code_name (const char *name)
{
    char *code = NULL;
    int count = 0;
    while (locale_data[count].name != '\0') {
        if (!strncmp (locale_data[count].name, name, strlen (name))) {
            code = strdup(locale_data[count].code);
            return code;
        }
        count++;
    }
    return code;
}

/* kbd utils */
void
apply_kbd_config (const char *kbd)
{
    char cmd[100];
    sprintf (cmd, "setxkbmap %s", kbd);
    Utils::runCmd(cmd);
}

/* get system kbd code in CURRENTKBDFILE
 * then convert it into its name with KBDFILE */
string get_kbd_name()
{
    /* Sorry for this */
    bool proceed = false, over = false;
    string layout, variant, code;
    string words[2];
    ifstream fichier(CURRENTKBDFILE);
        if ( fichier )
        {
            string line;
            while ( getline( fichier, line ))
            {
                str_parse(line,'"',words);
                if(words[0] == "XKBLAYOUT=")
                {
                    string words2[20];
                    ifstream fichier2(KBDFILE);
                    if ( fichier2 )
                    {
                        string line;
                        while ( getline( fichier2, line ) && !over)
                        {
                            str_parse(line,' ',words2);
                            if(words2[1] == "layout")
                            {
                                proceed =true;
                                continue;
                            }
                            if(words2[1] == "variant")
                            {
                                proceed = false;
                                over = true;
                            }
                            if(words2[0] == words[1] && proceed)
                            {
                                code = words2[0];
                                layout = words2[1];
                                for(int i=2; i<12; i++)
                                {
                                    if(words2[i] != "")
                                        layout = layout + " " + words2[i];
                                }
                            }
                            for(int i=0; i<12;i++)
                                words2[i] = "";
                        }
                    }
                }
                proceed = false;
                over = false;
                if(words[0] == "XKBVARIANT=")
                {
                    string words2[20];
                    ifstream fichier2(KBDFILE);
                    if ( fichier2 )
                    {
                        string line;
                        while ( getline( fichier2, line ) && !over)
                        {
                            str_parse(line,' ',words2);
                            if(words2[1] == "variant")
                            {
                                proceed =true;
                                continue;
                            }
                            if(words2[1] == "option")
                            {
                                proceed = false;
                                over = true;
                            }
                            if((words2[0] == words[1]) && proceed && (code + ":" == words2[1]))
                            {
                                variant = words2[2];
                                for(int i=3; i<12; i++)
                                {
                                    if(words2[i] != "")
                                        variant = variant + " " + words2[i];
                                }
                                return variant;
                            }
                            for(int i=0; i<12;i++)
                                words2[i] = "";
                        }
                    }
                }
                words[1] = "";
            }
            return layout;
        }
    return NULL;
}

/* get appstarter infos : autostarted program name and icon
 * then put it into pushB_web */
void
get_autostart_info(QPushButton *btn)
{
    string application,command,icon;

    string words[2];
#ifndef __arm__
    ifstream fichier("/home/linutop/.config/appstarterrc");
#else
    ifstream fichier("/home/pi/.config/appstarterrc");
#endif
    if ( fichier.fail() )
    {
        fichier.close();
        fichier.open("/etc/xdg/appstarterrc");
    }
    if ( fichier )
    {
        string line;
        bool parse = false;
        while ( getline( fichier, line ))
        {
            if(line == "[default]")
                parse = true;
            if(line == "")
                parse = false;
            if(parse)
            {
                str_parse(line,'=',words);
                if(words[0] == "selected")
                {
                    application = words[1];
                }
                if(words[0] == "command")
                {
                    command = words[1];
                }
            }
        }

        if((application == "none") || ((application == "firefox") && (command == "true")))
        {
            command = "None";
            icon = "";
        }
        else
        {
            string words2[2],file;
            file = "/usr/share/appstarter/plugins/" + application + "-plugin.desktop";
            ifstream fichier2(file.c_str());
            if ( fichier2 )
            {
                string line;
                while ( getline( fichier2, line ) )
                {
                    str_parse(line,'=',words2);
                    if(words2[0] == "GenericName")
                    {
                        command = words2[1];
                    }
                    if(words2[0] == "Icon")
                    {
                        icon = words2[1];
                    }
                }
            }
        }

        btn->setText(QString::fromStdString(command));
        btn->setIcon(QIcon(QString::fromStdString(icon)));
        btn->setIconSize(QSize(20,20));
    }
}

#define SCHEDFILE "/etc/linutopd/scheduler/jobs"
/* set reboot configuration */
bool
reboot_scheduled (unsigned int *hours, unsigned int *minutes)
{
    bool ret = false;
    string words[5];
    ifstream fichier(SCHEDFILE);
        if ( fichier )
        {
            string line;
            while ( getline( fichier, line ) )
            {
                str_parse(line,' ',words);
                if(words[2] == "reboot")
                {
                    *hours = atoi (words[1].c_str());
                    *minutes = atoi (words[0].c_str());
                    ret = true;
                }
            }
        }

    return ret;
}

/* parse filename into a string */
string
file_get_contents(const char *filename)
{
    ifstream in(filename);
    if (in.fail())
    {
        return NULL;
    }

    std::stringstream buffer;
    buffer << in.rdbuf();
    in.close();
    return buffer.str();
}

/* parse the string str into ret with del as delimiter */
void
str_parse(string str,char del,string *ret)
{
    istringstream iss(str);
    string mot;
    int i=0;
    while ( std::getline( iss, mot,del) )
    {
        if(mot == "")
            continue;
        ret[i] = mot;
        i++;
    }
}

QString get_kbd_name_from_code(QString kbd_code)
{
    bool proceed = false, over = false;
    QString ret;
    QFile fichier(KBDFILE);
        if ( fichier.open(QIODevice::ReadOnly | QIODevice::Text) )
        {
            QStringList tabTmp;
            QString line;
            QTextStream in(&fichier);
            while ( !in.atEnd() && !over)
            {
                line = in.readLine();
                tabTmp = line.split(" ",QString::SkipEmptyParts);
                if(tabTmp.size() >= 2)
                {
                    if(tabTmp[1] == "layout")
                    {
                        proceed =true;
                        continue;
                    }
                    if(tabTmp[1] == "variant")
                    {
                        proceed = false;
                        over = true;
                    }
                    if(proceed && tabTmp[0] == kbd_code)
                    {
                        ret = tabTmp[1];
                        for(int i=2; i<tabTmp.size(); i++)
                        {
                            ret += tabTmp[i];
                        }
                    }
                }
            }
        }
        return ret;
}

QString Utils::getIconTheme()
{
    QProcess p;
    p.start("qdbus --qt=qt4 org.xfce.Xfconf /org/xfce/Xfconf GetProperty xsettings /Net/IconThemeName");
    p.waitForFinished();
    QString r(p.readAllStandardOutput());
    return r.simplified();
}
